@@ -5,8 +5,9 @@
 #ifndef __xio_fdnum_h_included
 #define __xio_fdnum_h_included 1
 
-extern const struct addrdesc addr_fd;
+extern const struct addrdesc xioaddr_fd;
+extern const struct addrdesc xioaddr_accept_fd;
 
-extern int xioopen_fd(struct opt *opts, int rw, xiosingle_t *xfd, int numfd, int dummy2, int dummy3);
+extern int xioopen_fd(struct opt *opts, int rw, xiosingle_t *xfd, int numfd);
 
 #endif /* !defined(__xio_fdnum_h_included) */
