@@ -11,9 +11,10 @@
 #include "xioopen.h"
 #include "xio-ascii.h"
 #include "xio-socket.h"
-#include "xio-ip.h"	/* xiogetaddrinfo() */
+#include "xio-ip.h"	/* xioresolve() */
 
 #include "xio-ip6.h"
+#include "nestlex.h"
 
 
 static char *inet6addr_info(const struct in6_addr *sa, char *buff, size_t blen);
@@ -22,8 +23,11 @@ static char *inet6addr_info(const struct in6_addr *sa, char *buff, size_t blen);
 #ifdef IPV6_V6ONLY
 const struct optdesc opt_ipv6_v6only = { "ipv6-v6only", "ipv6only", OPT_IPV6_V6ONLY, GROUP_SOCK_IP6, PH_PREBIND, TYPE_INT, OFUNC_SOCKOPT, SOL_IPV6, IPV6_V6ONLY };
 #endif
-#ifdef IPV6_JOIN_GROUP
-const struct optdesc opt_ipv6_join_group = { "ipv6-join-group", "join-group", OPT_IPV6_JOIN_GROUP, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_IP_MREQN, OFUNC_SOCKOPT, SOL_IPV6, IPV6_JOIN_GROUP };
+#if defined(HAVE_STRUCT_IP_MREQ) || defined(HAVE_STRUCT_IP_MREQN)
+const struct optdesc opt_ipv6_join_group = { "ipv6-join-group", "join-group", OPT_IPV6_JOIN_GROUP, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_IP_MREQN, OFUNC_SPEC, SOL_IPV6, IPV6_JOIN_GROUP };
+#endif
+#ifdef MCAST_JOIN_SOURCE_GROUP
+const struct optdesc opt_ipv6_join_source_group = { "ipv6-join-source-group", "join-source-group", OPT_IPV6_JOIN_SOURCE_GROUP, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_GROUP_SOURCE_REQ, OFUNC_SPEC, SOL_IPV6, MCAST_JOIN_SOURCE_GROUP };
 #endif
 #ifdef IPV6_PKTINFO
 const struct optdesc opt_ipv6_pktinfo = { "ipv6-pktinfo", "pktinfo", OPT_IPV6_PKTINFO, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_INT, OFUNC_SOCKOPT, SOL_IPV6, IPV6_PKTINFO };
@@ -75,7 +79,41 @@ const struct optdesc opt_ipv6_recvtclass = { "ipv6-recvtclass", "recvtclass", OP
 const struct optdesc opt_ipv6_recvpathmtu = { "ipv6-recvpathmtu", "recvpathmtu", OPT_IPV6_RECVPATHMTU, GROUP_SOCK_IP6, PH_PASTSOCKET, TYPE_INT, OFUNC_SOCKOPT, SOL_IPV6, IPV6_RECVPATHMTU };
 #endif
 
-int xioparsenetwork_ip6(const char *rangename, struct xiorange *range) {
+/* Returns canonical form of IPv6 address.
+   IPv6 address may be enclose in brackets.
+   Returns STAT_OK on success, STAT_NORETRY on failure. */
+int xioip6_pton(
+	const char *src,
+	struct in6_addr *dst,
+	const int ai_flags[2])
+{
+   union sockaddr_union sockaddr;
+   socklen_t sockaddrlen = sizeof(sockaddr);
+
+   if (src[0] == '[') {
+      char plainaddr[INET6_ADDRSTRLEN];
+      char *clos;
+
+      strncpy(plainaddr, src+1, INET6_ADDRSTRLEN);
+      plainaddr[INET6_ADDRSTRLEN-1] = '\0';
+      if ((clos = strchr(plainaddr, ']')) != NULL)
+	 *clos = '\0';
+      return xioip6_pton(plainaddr, dst, ai_flags);
+   }
+   if (xioresolve(src, NULL, PF_INET6, 0, 0, &sockaddr, &sockaddrlen,
+		  ai_flags)
+       != STAT_OK) {
+      return STAT_NORETRY;
+   }
+   *dst = sockaddr.ip6.sin6_addr;
+   return STAT_OK;
+}
+
+int xioparsenetwork_ip6(
+	const char *rangename,
+	struct xiorange *range,
+	const int ai_flags[2])
+{
    char *delimpos;	/* absolute address of delimiter */
    size_t delimind;	/* index of delimiter in string */
    unsigned int bits;	/* netmask bits */
@@ -104,8 +142,8 @@ int xioparsenetwork_ip6(const char *rangename, struct xiorange *range) {
       return STAT_NORETRY;
    }
    baseaddr[delimind-2] = '\0';
-   if (xiogetaddrinfo(baseaddr, NULL, PF_INET6, 0, 0, &sockaddr, &sockaddrlen,
-		      0, 0)
+   if (xioresolve(baseaddr, NULL, PF_INET6, 0, 0, &sockaddr, &sockaddrlen,
+		  ai_flags)
        != STAT_OK) {
       return STAT_NORETRY;
    }
@@ -202,7 +240,7 @@ int xiocheckrange_ip6(struct sockaddr_in6 *pa, struct xiorange *range) {
        masked.u6_addr32[1] != rangeaddr->u6_addr32[1] ||
        masked.u6_addr32[2] != rangeaddr->u6_addr32[2] ||
        masked.u6_addr32[3] != rangeaddr->u6_addr32[3]) {
-      Debug1("client address %s is not permitted", peername); 
+      Debug1("client address %s is not permitted", peername);
       return -1;
    }
    return 0;
@@ -211,7 +249,7 @@ int xiocheckrange_ip6(struct sockaddr_in6 *pa, struct xiorange *range) {
 
 #if defined(HAVE_STRUCT_CMSGHDR) && defined(CMSG_DATA)
 /* provides info about the ancillary message:
-   converts the ancillary message in *cmsg into a form useable for further
+   converts the ancillary message in *cmsg into a form usable for further
    processing. knows the specifics of common message types.
    returns the number of resulting syntax elements in *num
    returns a sequence of \0 terminated type strings in *typbuff
@@ -220,11 +258,15 @@ int xiocheckrange_ip6(struct sockaddr_in6 *pa, struct xiorange *range) {
    the respective len parameters specify the available space in the buffers
    returns STAT_OK on success
  */
-int xiolog_ancillary_ip6(struct cmsghdr *cmsg, int *num,
-			 char *typbuff, int typlen,
-			 char *nambuff, int namlen,
-			 char *envbuff, int envlen,
-			 char *valbuff, int vallen) {
+int xiolog_ancillary_ip6(
+	struct single *sfd,
+	struct cmsghdr *cmsg,
+	int *num,
+	char *typbuff, int typlen,
+	char *nambuff, int namlen,
+	char *envbuff, int envlen,
+	char *valbuff, int vallen)
+{
    char scratch1[42];	/* can hold an IPv6 address in ASCII */
    char scratch2[32];
    size_t msglen;
@@ -431,4 +473,199 @@ xiosetsockaddrenv_ip6(int idx, char *namebuff, size_t namelen,
    return -1;
 }
 
+
+#if defined(HAVE_STRUCT_IPV6_MREQ)
+int xioapply_ipv6_join_group(
+	struct single *sfd,
+	struct opt *opt)
+{
+	struct ipv6_mreq ip6_mreq = {{{{0}}}};
+	union sockaddr_union sockaddr1;
+	socklen_t socklen1 = sizeof(sockaddr1.ip6);
+	int res;
+
+	/* Always two parameters */
+	/* First parameter is multicast address */
+	if ((res =
+	     xioresolve(opt->value.u_string/*multiaddr*/, NULL,
+			sfd->para.socket.la.soa.sa_family,
+			SOCK_DGRAM, IPPROTO_IP,
+			&sockaddr1, &socklen1,
+			sfd->para.socket.ip.ai_flags))
+	    != STAT_OK) {
+	   return res;
+	}
+	ip6_mreq.ipv6mr_multiaddr = sockaddr1.ip6.sin6_addr;
+	if (ifindex(opt->value2.u_string/*param2*/,
+		    &ip6_mreq.ipv6mr_interface, -1)
+	    < 0) {
+		Error1("interface \"%s\" not found",
+		       opt->value2.u_string/*param2*/);
+		ip6_mreq.ipv6mr_interface = htonl(0);
+	}
+
+	if (Setsockopt(sfd->fd, opt->desc->major, opt->desc->minor,
+		       &ip6_mreq, sizeof(ip6_mreq)) < 0) {
+		Error6("setsockopt(%d, %d, %d, {...,0x%08x}, "F_Zu"): %s",
+		       sfd->fd, opt->desc->major, opt->desc->minor,
+		       ip6_mreq.ipv6mr_interface,
+		       sizeof(ip6_mreq),
+		       strerror(errno));
+		opt->desc = ODESC_ERROR;
+		return -1;
+	}
+	return 0;
+}
+#endif /* defined(HAVE_STRUCT_IPV6_MREQ) */
+
+#if HAVE_STRUCT_GROUP_SOURCE_REQ
+int xiotype_ip6_join_source_group(
+	char *token, const struct optname *ent, struct opt *opt)
+{
+   /* We do not resolve the addresses here because we do not yet know
+      if we are coping with an IPv4 or IPv6 socat address */
+   const char *ends[] = { ":", NULL };
+   const char *nests[] = { "[","]", NULL };
+   char buff[512], *buffp=buff; size_t bufspc = sizeof(buff)-1;
+   char *tokp = token;
+   int parsres;
+
+   /* Parse first IP address (mcast group), expect ':' */
+   parsres =
+      nestlex((const char **)&tokp, &buffp, &bufspc,
+	      ends, NULL, NULL, nests,
+	      true, false, false);
+   if (parsres < 0) {
+      Error1("option too long:  \"%s\"", token);
+      return -1;
+   } else if (parsres > 0) {
+      Error1("syntax error in \"%s\"", token);
+      return -1;
+   }
+   if (*tokp != ':') {
+      Error1("syntax in option %s: missing ':'", token);
+   }
+   *buffp++ = '\0';
+   if ((opt->value.u_string/*mcaddr*/ = strdup(buff)) == NULL) {
+      int _errno = errno;
+      Error1("strdup(\"%s\"): out of memory", buff);
+      errno = _errno;
+      return -1;
+   }
+
+   ++tokp;
+   /* Parse interface name/index, expect ':' or '\0'' */
+   buffp = buff;
+   parsres =
+      nestlex((const char **)&tokp, &buffp, &bufspc,
+	      ends, NULL, NULL, nests,
+	      true, false, false);
+   if (parsres < 0) {
+      Error1("option too long:  \"%s\"", token);
+      return -1;
+   } else if (parsres > 0) {
+      Error1("syntax error in \"%s\"", token);
+      return -1;
+   }
+   if (*tokp != ':') {
+      Error1("syntax in option %s: missing ':'", token);
+   }
+   *buffp++ = '\0';
+   if ((opt->value2.u_string/*ifindex*/ = Malloc(IF_NAMESIZE)) == NULL) {
+      int _errno = errno;
+      free(opt->value.u_string);
+      errno = _errno;
+      return -1;
+   }
+   strncpy(opt->value2.u_string/*ifindex*/, buff, IF_NAMESIZE);
+
+   ++tokp;
+   /* Parse second IP address (source address), expect ':' or '\0'' */
+   buffp = buff;
+   parsres =
+      nestlex((const char **)&tokp, &buffp, &bufspc,
+	      ends, NULL, NULL, nests,
+	      true, false, false);
+   if (parsres < 0) {
+      Error1("option too long:  \"%s\"", token);
+      return -1;
+   } else if (parsres > 0) {
+      Error1("syntax error in \"%s\"", token);
+      return -1;
+   }
+   if (*tokp) {
+      Error1("syntax in option %s: trailing cruft", token);
+   }
+   *buffp++ = '\0';
+   if ((opt->value3.u_string/*srcaddr*/ = strdup(buff)) == NULL) {
+      int _errno = errno;
+      Error1("strdup(\"%s\"): out of memory", buff);
+      free(opt->value.u_string);
+      errno = _errno;
+      return -1;
+   }
+
+   Info4("setting option \"%s\" to {\"%s\",\"%s\",\"%s\"}",
+	 ent->desc->defname,
+	 opt->value.u_string/*mcaddr*/,
+	 opt->value2.u_string/*ifindex*/,
+	 opt->value3.u_string/*srcaddr*/);
+
+   if (!xioparms.experimental) {
+      Warn1("option %s is experimental", opt->desc->defname);
+   }
+
+   return 0;
+}
+
+int xioapply_ip6_join_source_group(struct single *sfd, struct opt *opt) {
+   struct group_source_req ip6_gsr = {0};
+   union sockaddr_union sockaddr1;
+   socklen_t socklen1 = sizeof(sockaddr1.ip6);
+   union sockaddr_union sockaddr2;
+   socklen_t socklen2 = sizeof(sockaddr2.ip6);
+   int res;
+
+   /* First parameter is always multicast address */
+   if ((res =
+	xioresolve(opt->value.u_string/*mcaddr*/, NULL,
+		   sfd->para.socket.la.soa.sa_family,
+		   SOCK_DGRAM, IPPROTO_IP, &sockaddr1, &socklen1,
+		   sfd->para.socket.ip.ai_flags))
+       != STAT_OK) {
+      return res;
+   }
+   memcpy(&ip6_gsr.gsr_group, &sockaddr1.ip6, socklen1);
+   /* Second parameter is interface name/index */
+   if (ifindex(opt->value2.u_string/*ifindex*/,
+	       &ip6_gsr.gsr_interface, -1)
+       < 0) {
+      Error1("interface \"%s\" not found",
+	     opt->value.u_string/*ifindex*/);
+      ip6_gsr.gsr_interface = 0;
+   }
+   /* Third parameter is source address */
+   if ((res =
+	xioresolve(opt->value3.u_string/*srcaddr*/, NULL,
+		   sfd->para.socket.la.soa.sa_family,
+		   SOCK_DGRAM, IPPROTO_IP, &sockaddr2, &socklen2,
+		   sfd->para.socket.ip.ai_flags))
+       != STAT_OK) {
+      return res;
+   }
+   memcpy(&ip6_gsr.gsr_source, &sockaddr2.ip6, socklen2);
+   if (Setsockopt(sfd->fd, opt->desc->major, opt->desc->minor,
+		  &ip6_gsr, sizeof(ip6_gsr)) < 0) {
+      Error6("setsockopt(%d, %d, %d, {%d,...}, "F_Zu"): %s",
+	     sfd->fd, opt->desc->major, opt->desc->minor,
+	     ip6_gsr.gsr_interface,
+	     sizeof(ip6_gsr),
+	     strerror(errno));
+      opt->desc = ODESC_ERROR;
+      return -1;
+   }
+   return 0;
+}
+#endif /* HAVE_STRUCT_GROUP_SOURCE_REQ */
+
 #endif /* WITH_IP6 */
